import 'dart:math';

import 'package:article_flutter_riverpod/widgets/space.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

final countServiceProvider = StateNotifierProvider<CountService, int>((ref) => CountService(42)); // <1>

class CountService extends StateNotifier<int> {
  CountService(int firstValue) : super(firstValue);

  void increment() {
    state++;
  }
}

final randomProvider = Provider<int>((ref) => Random().nextInt(10)); // <2>

final multiplyByRandomProvider = Provider( // <3>
  (ref) => ref.watch(countServiceProvider) * ref.watch(randomProvider),
  dependencies: [
    countServiceProvider,
    randomProvider,
  ],
);

class ParentExample extends StatelessWidget {
  const ParentExample({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ProviderScope( // <4>
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          const Counter(), // <5>
          HSpace.xl,
          ProviderScope( // <6>
            overrides: [
              countServiceProvider.overrideWithValue(CountService(13)), // <7>
            ],
            child: const Counter(), // <8>
          ),
        ],
      ),
    );
  }
}

class Counter extends ConsumerStatefulWidget {
  const Counter({Key? key}) : super(key: key);

  @override
  ConsumerState<Counter> createState() => _CounterState();
}

class _CounterState extends ConsumerState<Counter> {
  late int firstValue;

  @override
  void initState() {
    super.initState();
    firstValue = ref.read(countServiceProvider); // <9>
  }

  @override
  Widget build(BuildContext context) {
    final currentValue = ref.watch(countServiceProvider); // <10>
    final random = ref.watch(randomProvider);
    final multiplyByRandom = ref.watch(multiplyByRandomProvider);

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Text("First value: $firstValue"),
        HSpace.s,
        Text("Current value: $currentValue"),
        HSpace.s,
        Text("Multiplied by $random: $multiplyByRandom"),
        HSpace.m,
        ElevatedButton(
          onPressed: _onIncrement,
          child: const Text("Increment"),
        )
      ],
    );
  }

  void _onIncrement() {
    ref.read(countServiceProvider.notifier).increment(); // <11>
  }
}
